@@ -22,7 +22,7 @@ package org.joni;
 import org.jcodings.ApplyAllCaseFoldFunction;
 import org.jcodings.Encoding;
 import org.joni.ast.CClassNode;
-import org.joni.ast.ConsAltNode;
+import org.joni.ast.ListNode;
 import org.joni.ast.StringNode;
 
 final class ApplyCaseFold implements ApplyAllCaseFoldFunction {
@@ -34,30 +34,44 @@ final class ApplyCaseFold implements ApplyAllCaseFoldFunction {
         ScanEnvironment env = arg.env;
         Encoding enc = env.enc;
         CClassNode cc = arg.cc;
+        CClassNode ascCc = arg.ascCc;
         BitSet bs = cc.bs;
+        boolean addFlag;
+
+        if (ascCc == null) {
+            addFlag = false;
+        } else if (Encoding.isAscii(from) == Encoding.isAscii(to[0])) {
+            addFlag = true;
+        } else {
+            addFlag = ascCc.isCodeInCC(enc, from);
+            if (ascCc.isNot()) addFlag = !addFlag;
+        }
 
         if (length == 1) {
             boolean inCC = cc.isCodeInCC(enc, from);
-
             if (Config.CASE_FOLD_IS_APPLIED_INSIDE_NEGATIVE_CCLASS) {
                 if ((inCC && !cc.isNot()) || (!inCC && cc.isNot())) {
-                    if (enc.minLength() > 1 || to[0] >= BitSet.SINGLE_BYTE_SIZE) {
-                        cc.addCodeRange(env, to[0], to[0]);
-                    } else {
-                        /* /(?i:[^A-C])/.match("a") ==> fail. */
-                        bs.set(to[0]);
+                    if (addFlag) {
+                        if (enc.minLength() > 1 || to[0] >= BitSet.SINGLE_BYTE_SIZE) {
+                            cc.addCodeRange(env, to[0], to[0], false);
+                        } else {
+                            /* /(?i:[^A-C])/.match("a") ==> fail. */
+                            bs.set(to[0]);
+                        }
                     }
                 }
             } else {
                 if (inCC) {
-                    if (enc.minLength() > 1 || to[0] >= BitSet.SINGLE_BYTE_SIZE) {
-                        if (cc.isNot()) cc.clearNotFlag(enc);
-                        cc.addCodeRange(env, to[0], to[0]);
-                    } else {
-                        if (cc.isNot()) {
-                            bs.clear(to[0]);
+                    if (addFlag) {
+                        if (enc.minLength() > 1 || to[0] >= BitSet.SINGLE_BYTE_SIZE) {
+                            if (cc.isNot()) cc.clearNotFlag(env);
+                            cc.addCodeRange(env, to[0], to[0], false);
                         } else {
-                            bs.set(to[0]);
+                            if (cc.isNot()) {
+                                bs.clear(to[0]);
+                            } else {
+                                bs.set(to[0]);
+                            }
                         }
                     }
                 }
@@ -76,12 +90,12 @@ final class ApplyCaseFold implements ApplyAllCaseFoldFunction {
                     node.catCode(to[i], enc);
                 }
 
-                ConsAltNode alt = ConsAltNode.newAltNode(node, null);
+                ListNode alt = ListNode.newAlt(node, null);
 
                 if (arg.tail == null) {
                     arg.altRoot = alt;
                 } else {
-                    arg.tail.setCdr(alt);
+                    arg.tail.setTail(alt);
                 }
                 arg.tail = alt;
             }
