@@ -1324,7 +1324,7 @@ class Lexer extends ScannerSupport {
 
     protected final void syntaxWarn(String message) {
         if (env.warnings != WarnCallback.NONE) {
-            env.warnings.warn(message + ": /" + new String(bytes, getBegin(), getEnd()) + "/");
+            env.warnings.warn(message + ": /" + new String(bytes, getBegin(), getEnd() - getBegin()) + "/");
         }
     }
 }
