@@ -24,11 +24,13 @@ import static org.joni.Config.USE_SUNDAY_QUICK_SEARCH;
 import static org.joni.Option.isCaptureGroup;
 import static org.joni.Option.isDontCaptureGroup;
 
+import java.nio.charset.Charset;
 import java.util.Collections;
 import java.util.Iterator;
 
 import org.jcodings.CaseFoldCodeItem;
 import org.jcodings.Encoding;
+import org.jcodings.EncodingDB;
 import org.jcodings.specific.ASCIIEncoding;
 import org.jcodings.specific.UTF8Encoding;
 import org.jcodings.util.BytesHash;
@@ -88,6 +90,18 @@ public final class Regex {
     byte[][]templates;                      /* fixed pattern strings not embedded in bytecode */
     int templateNum;
 
+    private static final Encoding DEFAULT_ENCODING;
+    static {
+        Encoding defaultEncoding;
+        EncodingDB.Entry entry = EncodingDB.getEncodings().get(Charset.defaultCharset().name().getBytes());
+        if (entry == null) {
+            defaultEncoding = UTF8Encoding.INSTANCE;
+        } else {
+            defaultEncoding = entry.getEncoding();
+        }
+        DEFAULT_ENCODING = defaultEncoding;
+    }
+
     public Regex(CharSequence cs) {
         this(cs.toString());
     }
@@ -164,7 +178,7 @@ public final class Regex {
     }
 
     public Matcher matcher(byte[]bytes, int p, int end) {
-        return factory.create(this, numMem == 0 ? null : new Region(numMem + 1), bytes, p, end);
+        return factory.create(this, numMem == 0 ? null : Region.newRegion(numMem + 1), bytes, p, end);
     }
 
     public Matcher matcherNoRegion(byte[]bytes, int p, int end) {
@@ -232,9 +246,13 @@ public final class Regex {
     }
 
     public int nameToBackrefNumber(byte[]name, int nameP, int nameEnd, Region region) {
+        return nameToBackrefNumber(name, nameP, nameEnd, DEFAULT_ENCODING, region);
+    }
+
+    public int nameToBackrefNumber(byte[]name, int nameP, int nameEnd, Encoding nameEncoding, Region region) {
         NameEntry e = nameToGroupNumbers(name, nameP, nameEnd);
         if (e == null) throw new ValueException(ErrorMessages.UNDEFINED_NAME_REFERENCE,
-                                                new String(name, nameP, nameEnd - nameP));
+                                                new String(name, nameP, nameEnd - nameP, nameEncoding.getCharset()));
 
         switch(e.backNum) {
         case 0:
@@ -244,7 +262,7 @@ public final class Regex {
         default:
             if (region != null) {
                 for (int i = e.backNum - 1; i >= 0; i--) {
-                    if (region.beg[e.backRefs[i]] != Region.REGION_NOTPOS) return e.backRefs[i];
+                    if (region.getBeg(e.backRefs[i]) != Region.REGION_NOTPOS) return e.backRefs[i];
                 }
             }
             return e.backRefs[e.backNum - 1];
@@ -352,7 +370,9 @@ public final class Regex {
             if (e.length >= 3 || (e.length >= 2 && allowReverse)) {
                 forward = enc.toLowerCaseTable() != null ? Search.SLOW_IC_SB_FORWARD : Search.SLOW_IC_FORWARD;
                 if (!setupBMSkipMap(true)) {
-                    forward = allowReverse ? Search.BM_IC_FORWARD : Search.BM_NOT_REV_IC_FORWARD;
+                    forward = allowReverse ? (enc.toLowerCaseTable() != null ? Search.SLOW_IC_SB_FORWARD : Search.SLOW_IC_FORWARD) : Search.BM_NOT_REV_IC_FORWARD;
+                    // FIXME: put above line in place to work around some failures.  Either BM_IC_FORWARD is broken here or we are choosing it when we shouldn't.
+                    //forward = allowReverse ? Search.BM_IC_FORWARD : Search.BM_NOT_REV_IC_FORWARD;
                 } else {
                     forward = enc.toLowerCaseTable() != null ? Search.SLOW_IC_SB_FORWARD : Search.SLOW_IC_FORWARD;
                 }
