@@ -28,7 +28,6 @@ import org.joni.exception.ValueException;
 
 abstract class ScannerSupport extends IntHolder implements ErrorMessages {
     protected final Encoding enc;       // fast access to encoding
-
     protected final byte[]bytes;        // pattern
     protected int p;                    // current scanner position
     protected int stop;                 // pattern end (mutable)
@@ -41,24 +40,20 @@ abstract class ScannerSupport extends IntHolder implements ErrorMessages {
 
     protected ScannerSupport(Encoding enc, byte[]bytes, int p, int end) {
         this.enc = enc;
-
         this.bytes = bytes;
         this.begin = p;
         this.end = end;
-
-        reset();
     }
 
-    protected int getBegin() {
+    protected final int getBegin() {
         return begin;
     }
 
-    protected int getEnd() {
+    protected final int getEnd() {
         return end;
     }
 
-    private final int INT_SIGN_BIT = 1 << 31;
-
+    private static final int INT_SIGN_BIT = 1 << 31;
     protected final int scanUnsignedNumber() {
         int last = c;
         int num = 0; // long ???
@@ -77,21 +72,23 @@ abstract class ScannerSupport extends IntHolder implements ErrorMessages {
         return num;
     }
 
-    protected final int scanUnsignedHexadecimalNumber(int maxLength) {
+    protected final int scanUnsignedHexadecimalNumber(int minLength, int maxLength) {
         int last = c;
         int num = 0;
+        int restLen = maxLength - minLength;
         while(left() && maxLength-- != 0) {
             fetch();
             if (enc.isXDigit(c)) {
-                int onum = num;
                 int val = enc.xdigitVal(c);
+                if ((Integer.MAX_VALUE - val) / 16 < num) return -1;
                 num = (num << 4) + val;
-                if (((onum ^ num) & INT_SIGN_BIT) != 0) return -1;
             } else {
                 unfetch();
+                maxLength++;
                 break;
             }
         }
+        if (maxLength > restLen) return -2;
         c = last;
         return num;
     }
