@@ -19,20 +19,18 @@
  */
 package org.joni.ast;
 
+import org.joni.Config;
 import org.joni.ScanEnvironment;
 import org.joni.exception.ErrorMessages;
 import org.joni.exception.ValueException;
 
 public final class BackRefNode extends StateNode {
-    //private static int NODE_BACKREFS_SIZE = 6;
-
-    //int state;
+    public final int back[];
     public int backNum;
-    public int back[];
-
     public int nestLevel;
 
-    public BackRefNode(int backNum, int[]backRefs, boolean byName, ScanEnvironment env) {
+    private BackRefNode(int backNum, int[]backRefs, boolean byName, ScanEnvironment env) {
+        super(BREF);
         this.backNum = backNum;
         if (byName) setNameRef();
 
@@ -42,45 +40,20 @@ public final class BackRefNode extends StateNode {
                 break;
             }
         }
-
-        back = new int[backNum];
-        System.arraycopy(backRefs, 0, back, 0, backNum); // shall we really dup it ???
+        back = backRefs;
     }
 
-    // #ifdef USE_BACKREF_AT_LEVEL
     public BackRefNode(int backNum, int[]backRefs, boolean byName, boolean existLevel, int nestLevel, ScanEnvironment env) {
         this(backNum, backRefs, byName, env);
 
-        if (existLevel) {
-            //state |= NST_NEST_LEVEL;
+        if (Config.USE_BACKREF_WITH_LEVEL && existLevel) {
             setNestLevel();
             this.nestLevel = nestLevel;
         }
     }
 
-    @Override
-    public int getType() {
-        return BREF;
-    }
-
-    @Override
-    public String getName() {
-        return "Back Ref";
-    }
-
-    @Override
-    public String toString(int level) {
-        StringBuilder value = new StringBuilder(super.toString(level));
-        value.append("\n  backNum: " + backNum);
-        String backs = "";
-        for (int i=0; i<back.length; i++) backs += back[i] + ", ";
-        value.append("\n  back: " + backs);
-        value.append("\n  nextLevel: " + nestLevel);
-        return value.toString();
-    }
-
     public void renumber(int[]map) {
-        if (!isNameRef()) throw new ValueException(ErrorMessages.ERR_NUMBERED_BACKREF_OR_CALL_NOT_ALLOWED);
+        if (!isNameRef()) throw new ValueException(ErrorMessages.NUMBERED_BACKREF_OR_CALL_NOT_ALLOWED);
 
         int oldNum = backNum;
 
@@ -95,4 +68,19 @@ public final class BackRefNode extends StateNode {
         backNum = pos;
     }
 
+    @Override
+    public String getName() {
+        return "Back Ref";
+    }
+
+    @Override
+    public String toString(int level) {
+        StringBuilder sb = new StringBuilder(super.toString(level));
+        sb.append("\n  backNum: " + backNum);
+        String backs = "";
+        for (int i=0; i<back.length; i++) backs += back[i] + ", ";
+        sb.append("\n  back: " + backs);
+        sb.append("\n  nextLevel: " + nestLevel);
+        return sb.toString();
+    }
 }
