@@ -23,11 +23,8 @@ import org.jcodings.CodeRange;
 import org.jcodings.Encoding;
 import org.jcodings.IntHolder;
 import org.jcodings.constants.CharacterType;
-import org.jcodings.exception.EncodingException;
-import org.jcodings.specific.ASCIIEncoding;
 import org.joni.BitSet;
 import org.joni.CodeRangeBuffer;
-import org.joni.Config;
 import org.joni.ScanEnvironment;
 import org.joni.exception.ErrorMessages;
 import org.joni.exception.InternalException;
@@ -35,24 +32,15 @@ import org.joni.exception.SyntaxException;
 import org.joni.exception.ValueException;
 
 public final class CClassNode extends Node {
-    private static final int FLAG_NCCLASS_NOT = 1<<0;
-    private static final int FLAG_NCCLASS_SHARE = 1<<1;
+    private static final int FLAG_NCCLASS_NOT = 1 << 0;
 
-    int flags;
+    private int flags;
     public final BitSet bs = new BitSet();  // conditional creation ?
     public CodeRangeBuffer mbuf;            /* multi-byte info or NULL */
 
-    private int ctype;                      // for hashing purposes
-    private Encoding enc;                   // ...
-
-
     // node_new_cclass
-    public CClassNode() {}
-
-    public CClassNode(int ctype, Encoding enc, boolean not, int sbOut, int[]ranges) {
-        this(not, sbOut, ranges);
-        this.ctype = ctype;
-        this.enc = enc;
+    public CClassNode() {
+        super(CCLASS);
     }
 
     public void clear() {
@@ -61,111 +49,91 @@ public final class CClassNode extends Node {
         mbuf = null;
     }
 
-    // node_new_cclass_by_codepoint_range, only used by shared Char Classes
-    public CClassNode(boolean not, int sbOut, int[]ranges) {
-        if (not) setNot();
-        // bs.clear();
-
-        if (sbOut > 0 && ranges != null) {
-            int n = ranges[0];
-            for (int i=0; i<n; i++) {
-                int from = ranges[i * 2 + 1];
-                int to = ranges[i * 2 + 2];
-                for (int j=from; j<=to; j++) {
-                    if (j >= sbOut) {
-                        setupBuffer(ranges);
-                        return;
-                    }
-                    bs.set(j);
-                }
-            }
-        }
-        setupBuffer(ranges);
-    }
-
-    @Override
-    public int getType() {
-        return CCLASS;
-    }
-
     @Override
     public String getName() {
         return "Character Class";
     }
 
-    @Override
-    public boolean equals(Object other) {
-        if (!(other instanceof CClassNode)) return false;
-        CClassNode cc = (CClassNode)other;
-        return ctype == cc.ctype && isNot() == cc.isNot() && enc == cc.enc;
-    }
-
-    @Override
-    public int hashCode() {
-        if (Config.USE_SHARED_CCLASS_TABLE) {
-            int hash = 0;
-            hash += ctype;
-            hash += enc.hashCode();
-            if (isNot()) hash++;
-            return hash + (hash >> 5);
-        } else {
-            return super.hashCode();
-        }
-    }
-
     @Override
     public String toString(int level) {
         StringBuilder value = new StringBuilder();
         value.append("\n  flags: " + flagsToString());
         value.append("\n  bs: " + pad(bs, level + 1));
         value.append("\n  mbuf: " + pad(mbuf, level + 1));
-
         return value.toString();
     }
 
     public String flagsToString() {
         StringBuilder flags = new StringBuilder();
         if (isNot()) flags.append("NOT ");
-        if (isShare()) flags.append("SHARE ");
         return flags.toString();
     }
 
-    private void setupBuffer(int[]ranges) {
-        if (ranges != null) {
-            if (ranges[0] == 0) return;
-            mbuf = new CodeRangeBuffer(ranges);
-        }
-    }
-
     public boolean isEmpty() {
         return mbuf == null && bs.isEmpty();
     }
 
-    public void addCodeRangeToBuf(int from, int to) {
-        mbuf = CodeRangeBuffer.addCodeRangeToBuff(mbuf, from, to);
+    void addCodeRangeToBuf(ScanEnvironment env, int from, int to) {
+        addCodeRangeToBuf(env, from, to, true);
     }
 
+    void addCodeRangeToBuf(ScanEnvironment env, int from, int to, boolean checkDup) {
+        mbuf = CodeRangeBuffer.addCodeRangeToBuff(mbuf, env, from, to, checkDup);
+    }
+
+    // add_code_range, be aware of it returning null!
     public void addCodeRange(ScanEnvironment env, int from, int to) {
-        mbuf = CodeRangeBuffer.addCodeRange(mbuf, env, from, to);
+        addCodeRange(env, from, to, true);
     }
 
-    public void addAllMultiByteRange(Encoding enc) {
-        mbuf = CodeRangeBuffer.addAllMultiByteRange(enc, mbuf);
+    public void addCodeRange(ScanEnvironment env, int from, int to, boolean checkDup) {
+        mbuf = CodeRangeBuffer.addCodeRange(mbuf, env, from, to, checkDup);
     }
 
-    public void clearNotFlag(Encoding enc) {
+    void addAllMultiByteRange(ScanEnvironment env) {
+        mbuf = CodeRangeBuffer.addAllMultiByteRange(env, mbuf);
+    }
+
+    public void clearNotFlag(ScanEnvironment env) {
         if (isNot()) {
             bs.invert();
-
-            if (!enc.isSingleByte()) {
-                mbuf = CodeRangeBuffer.notCodeRangeBuff(enc, mbuf);
+            if (!env.enc.isSingleByte()) {
+                mbuf = CodeRangeBuffer.notCodeRangeBuff(env, mbuf);
             }
             clearNot();
         }
     }
 
+    public int isOneChar() {
+        if (isNot()) return -1;
+        int c = -1;
+        if (mbuf != null) {
+            int[]range = mbuf.getCodeRange();
+            c = range[1];
+            if (range[0] == 1 && c == range[2]) {
+                if (c < BitSet.SINGLE_BYTE_SIZE && bs.at(c)) {
+                    c = -1;
+                }
+            } else {
+                return -1;
+            }
+        }
+
+        for (int i = 0; i < BitSet.BITSET_SIZE; i++) {
+            int b1 = bs.bits[i];
+            if (b1 != 0) {
+                if ((b1 & (b1 - 1)) == 0 && c == -1) {
+                    c = BitSet.BITS_IN_ROOM * i + Integer.bitCount(b1 - 1);
+                } else {
+                    return -1;
+                }
+            }
+        }
+        return c;
+    }
+
     // and_cclass
-    public void and(CClassNode other, Encoding enc) {
+    public void and(CClassNode other, ScanEnvironment env) {
         boolean not1 = isNot();
         BitSet bsr1 = bs;
         CodeRangeBuffer buf1 = mbuf;
@@ -198,14 +166,14 @@ public final class CClassNode extends Node {
 
         CodeRangeBuffer pbuf = null;
 
-        if (!enc.isSingleByte()) {
+        if (!env.enc.isSingleByte()) {
             if (not1 && not2) {
-                pbuf = CodeRangeBuffer.orCodeRangeBuff(enc, buf1, false, buf2, false);
+                pbuf = CodeRangeBuffer.orCodeRangeBuff(env, buf1, false, buf2, false);
             } else {
-                pbuf = CodeRangeBuffer.andCodeRangeBuff(buf1, not1, buf2, not2);
+                pbuf = CodeRangeBuffer.andCodeRangeBuff(buf1, not1, buf2, not2, env);
 
                 if (not1) {
-                    pbuf = CodeRangeBuffer.notCodeRangeBuff(enc, pbuf);
+                    pbuf = CodeRangeBuffer.notCodeRangeBuff(env, pbuf);
                 }
             }
             mbuf = pbuf;
@@ -214,7 +182,7 @@ public final class CClassNode extends Node {
     }
 
     // or_cclass
-    public void or(CClassNode other, Encoding enc) {
+    public void or(CClassNode other, ScanEnvironment env) {
         boolean not1 = isNot();
         BitSet bsr1 = bs;
         CodeRangeBuffer buf1 = mbuf;
@@ -245,14 +213,14 @@ public final class CClassNode extends Node {
             bs.invert();
         }
 
-        if (!enc.isSingleByte()) {
+        if (!env.enc.isSingleByte()) {
             CodeRangeBuffer pbuf = null;
             if (not1 && not2) {
-                pbuf = CodeRangeBuffer.andCodeRangeBuff(buf1, false, buf2, false);
+                pbuf = CodeRangeBuffer.andCodeRangeBuff(buf1, false, buf2, false, env);
             } else {
-                pbuf = CodeRangeBuffer.orCodeRangeBuff(enc, buf1, not1, buf2, not2);
+                pbuf = CodeRangeBuffer.orCodeRangeBuff(env, buf1, not1, buf2, not2);
                 if (not1) {
-                    pbuf = CodeRangeBuffer.notCodeRangeBuff(enc, pbuf);
+                    pbuf = CodeRangeBuffer.notCodeRangeBuff(env, pbuf);
                 }
             }
             mbuf = pbuf;
@@ -260,109 +228,101 @@ public final class CClassNode extends Node {
     }
 
     // add_ctype_to_cc_by_range // Encoding out!
-    public void addCTypeByRange(int ctype, boolean not, Encoding enc, int sbOut, int mbr[]) {
+    public void addCTypeByRange(int ctype, boolean not, ScanEnvironment env, int sbOut, int mbr[]) {
         int n = mbr[0];
+        int i;
 
         if (!not) {
-            for (int i=0; i<n; i++) {
+            for (i=0; i<n; i++) {
                 for (int j=CR_FROM(mbr, i); j<=CR_TO(mbr, i); j++) {
                     if (j >= sbOut) {
-                        if (Config.VANILLA) {
-                            if (j == CR_TO(mbr, i)) {
-                                i++;
-                            } else if (j > CR_FROM(mbr, i)) {
-                                addCodeRangeToBuf(j, CR_TO(mbr, i));
-                                i++;
-                            }
-                        } else {
-                            if (j >= CR_FROM(mbr, i)) {
-                                addCodeRangeToBuf(j, CR_TO(mbr, i));
-                                i++;
-                            }
+                        if (j > CR_FROM(mbr, i)) {
+                            addCodeRangeToBuf(env, j, CR_TO(mbr, i));
+                            i++;
                         }
                         // !goto sb_end!, remove duplication!
                         for (; i<n; i++) {
-                            addCodeRangeToBuf(CR_FROM(mbr, i), CR_TO(mbr, i));
+                            addCodeRangeToBuf(env, CR_FROM(mbr, i), CR_TO(mbr, i));
                         }
                         return;
                     }
-                    bs.set(j);
+                    bs.set(env, j);
                 }
             }
             // !sb_end:!
-            for (int i=0; i<n; i++) {
-                addCodeRangeToBuf(CR_FROM(mbr, i), CR_TO(mbr, i));
+            for (; i<n; i++) {
+                addCodeRangeToBuf(env, CR_FROM(mbr, i), CR_TO(mbr, i));
             }
 
         } else {
             int prev = 0;
 
-            for (int i=0; i<n; i++) {
+            for (i=0; i<n; i++) {
                 for (int j=prev; j < CR_FROM(mbr, i); j++) {
                     if (j >= sbOut) {
                         // !goto sb_end2!, remove duplication
                         prev = sbOut;
                         for (i=0; i<n; i++) {
-                            if (prev < CR_FROM(mbr, i)) addCodeRangeToBuf(prev, CR_FROM(mbr, i) - 1);
+                            if (prev < CR_FROM(mbr, i)) addCodeRangeToBuf(env, prev, CR_FROM(mbr, i) - 1);
                             prev = CR_TO(mbr, i) + 1;
                         }
-                        if (prev < 0x7fffffff/*!!!*/) addCodeRangeToBuf(prev, 0x7fffffff);
+                        if (prev < 0x7fffffff/*!!!*/) addCodeRangeToBuf(env, prev, 0x7fffffff);
                         return;
                     }
-                    bs.set(j);
+                    bs.set(env, j);
                 }
                 prev = CR_TO(mbr, i) + 1;
             }
 
             for (int j=prev; j<sbOut; j++) {
-                bs.set(j);
+                bs.set(env, j);
             }
 
             // !sb_end2:!
             prev = sbOut;
-            for (int i=0; i<n; i++) {
-                if (prev < CR_FROM(mbr, i)) addCodeRangeToBuf(prev, CR_FROM(mbr, i) - 1);
+            for (i=0; i<n; i++) {
+                if (prev < CR_FROM(mbr, i)) addCodeRangeToBuf(env, prev, CR_FROM(mbr, i) - 1);
                 prev = CR_TO(mbr, i) + 1;
             }
-            if (prev < 0x7fffffff/*!!!*/) addCodeRangeToBuf(prev, 0x7fffffff);
+            if (prev < 0x7fffffff/*!!!*/) addCodeRangeToBuf(env, prev, 0x7fffffff);
         }
     }
 
-    public void addCType(int ctype, boolean not, ScanEnvironment env, IntHolder sbOut) {
+    private static int CR_FROM(int[] range, int i) {
+        return range[(i * 2) + 1];
+    }
+
+    private static int CR_TO(int[] range, int i) {
+        return range[(i * 2) + 2];
+    }
+
+    // add_ctype_to_cc
+    public void addCType(int ctype, boolean not, boolean asciiRange, ScanEnvironment env, IntHolder sbOut) {
         Encoding enc = env.enc;
-
-        if (Config.NON_UNICODE_SDW) {
-            switch(ctype) {
-            case CharacterType.S:
-                if (!not && env.syntax.op2OptionECMAScript()) {
-                    // treat \u2028 and \u2029 as whitespace
-                    addCodeRange(env, 8232, 8233);
-                }
-            case CharacterType.D:
-            case CharacterType.W:
-                ctype ^= CharacterType.SPECIAL_MASK;
-                if (not) {
-                    for (int c = 0; c < BitSet.SINGLE_BYTE_SIZE; c++) {
-                        if (!ASCIIEncoding.INSTANCE.isCodeCType(c, ctype)) bs.set(c);
-                        //if ((AsciiTables.AsciiCtypeTable[c] & (1 << ctype)) == 0) bs.set(c);
-                    }
-                    addAllMultiByteRange(enc);
-                } else {
-                    for (int c = 0; c < BitSet.SINGLE_BYTE_SIZE; c++) {
-                        if (ASCIIEncoding.INSTANCE.isCodeCType(c, ctype)) bs.set(c);
-                        //if ((AsciiTables.AsciiCtypeTable[c] & (1 << ctype)) != 0) bs.set(c);
-                    }
-                }
-                return;
-            }
-        }
-
         int[]ranges = enc.ctypeCodeRange(ctype, sbOut);
         if (ranges != null) {
-            addCTypeByRange(ctype, not, enc, sbOut.value, ranges);
+            if (asciiRange) {
+                CClassNode ccWork = new CClassNode();
+                ccWork.addCTypeByRange(ctype, not, env, sbOut.value, ranges);
+                if (not) {
+                    ccWork.addCodeRangeToBuf(env, 0x80, CodeRangeBuffer.LAST_CODE_POINT, false);
+                } else {
+                    CClassNode ccAscii = new CClassNode();
+                    if (enc.minLength() > 1) {
+                        ccAscii.addCodeRangeToBuf(env, 0x00, 0x7F);
+                    } else {
+                        ccAscii.bs.setRange(env, 0x00, 0x7F);
+                    }
+                    ccWork.and(ccAscii, env);
+                }
+                or(ccWork, env);
+            } else {
+                addCTypeByRange(ctype, not, env, sbOut.value, ranges);
+            }
             return;
         }
 
+        int maxCode = asciiRange ? 0x80 : BitSet.SINGLE_BYTE_SIZE;
         switch(ctype) {
         case CharacterType.ALPHA:
         case CharacterType.BLANK:
@@ -377,12 +337,12 @@ public final class CClassNode extends Node {
         case CharacterType.ALNUM:
             if (not) {
                 for (int c=0; c<BitSet.SINGLE_BYTE_SIZE; c++) {
-                    if (!enc.isCodeCType(c, ctype)) bs.set(c);
+                    if (!enc.isCodeCType(c, ctype)) bs.set(env, c);
                 }
-                addAllMultiByteRange(enc);
+                addAllMultiByteRange(env);
             } else {
                 for (int c=0; c<BitSet.SINGLE_BYTE_SIZE; c++) {
-                    if (enc.isCodeCType(c, ctype)) bs.set(c);
+                    if (enc.isCodeCType(c, ctype)) bs.set(env, c);
                 }
             }
             break;
@@ -391,35 +351,34 @@ public final class CClassNode extends Node {
         case CharacterType.PRINT:
             if (not) {
                 for (int c=0; c<BitSet.SINGLE_BYTE_SIZE; c++) {
-                    if (!enc.isCodeCType(c, ctype)) bs.set(c);
+                    if (!enc.isCodeCType(c, ctype) || c >= maxCode) bs.set(env, c);
                 }
+                if (asciiRange) addAllMultiByteRange(env);
             } else {
-                for (int c=0; c<BitSet.SINGLE_BYTE_SIZE; c++) {
-                    if (enc.isCodeCType(c, ctype)) bs.set(c);
+                for (int c=0; c<maxCode; c++) {
+                    if (enc.isCodeCType(c, ctype)) bs.set(env, c);
                 }
-                addAllMultiByteRange(enc);
+                if (!asciiRange) addAllMultiByteRange(env);
             }
             break;
 
         case CharacterType.WORD:
             if (!not) {
-                for (int c=0; c<BitSet.SINGLE_BYTE_SIZE; c++) {
-                    if (enc.isSbWord(c)) bs.set(c);
+                for (int c=0; c<maxCode; c++) {
+                    if (enc.isSbWord(c)) bs.set(env, c);
                 }
-
-                addAllMultiByteRange(enc);
+                if (!asciiRange) addAllMultiByteRange(env);
             } else {
                 for (int c=0; c<BitSet.SINGLE_BYTE_SIZE; c++) {
-                    try {
-                        if (enc.codeToMbcLength(c) > 0 && /* check invalid code point */
-                                !enc.isWord(c)) bs.set(c);
-                    } catch (EncodingException ve) {};
+                    if (enc.codeToMbcLength(c) > 0 && /* check invalid code point */
+                            !(enc.isWord(c) || c >= maxCode)) bs.set(env, c);
                 }
+                if (asciiRange) addAllMultiByteRange(env);
             }
             break;
 
         default:
-            throw new InternalException(ErrorMessages.ERR_PARSER_BUG);
+            throw new InternalException(ErrorMessages.PARSER_BUG);
         } // switch
     }
 
@@ -437,71 +396,79 @@ public final class CClassNode extends Node {
     }
 
     public static final class CCStateArg {
-        public int v;
-        public int vs;
-        public boolean vsIsRaw;
-        public boolean vIsRaw;
+        public int from;
+        public int to;
+        public boolean fromIsRaw;
+        public boolean toIsRaw;
         public CCVALTYPE inType;
         public CCVALTYPE type;
         public CCSTATE state;
     }
 
-    public void nextStateClass(CCStateArg arg, ScanEnvironment env) {
-        if (arg.state == CCSTATE.RANGE) throw new SyntaxException(ErrorMessages.ERR_CHAR_CLASS_VALUE_AT_END_OF_RANGE);
+    public void nextStateClass(CCStateArg arg, CClassNode ascCC, ScanEnvironment env) {
+        if (arg.state == CCSTATE.RANGE) throw new SyntaxException(ErrorMessages.CHAR_CLASS_VALUE_AT_END_OF_RANGE);
 
         if (arg.state == CCSTATE.VALUE && arg.type != CCVALTYPE.CLASS) {
             if (arg.type == CCVALTYPE.SB) {
-                bs.set(arg.vs);
+                bs.set(env, arg.from);
+                if (ascCC != null) ascCC.bs.set(arg.from);
             } else if (arg.type == CCVALTYPE.CODE_POINT) {
-                addCodeRange(env, arg.vs, arg.vs);
+                addCodeRange(env, arg.from, arg.from);
+                if (ascCC != null) ascCC.addCodeRange(env, arg.from, arg.from, false);
             }
         }
         arg.state = CCSTATE.VALUE;
         arg.type = CCVALTYPE.CLASS;
     }
 
-    public void nextStateValue(CCStateArg arg, ScanEnvironment env) {
-
+    public void nextStateValue(CCStateArg arg, CClassNode ascCc, ScanEnvironment env) {
         switch(arg.state) {
         case VALUE:
             if (arg.type == CCVALTYPE.SB) {
-                if (arg.vs > 0xff) throw new ValueException(ErrorMessages.ERR_INVALID_CODE_POINT_VALUE);
-                bs.set(arg.vs);
+                bs.set(env, arg.from);
+                if (ascCc != null) ascCc.bs.set(arg.from);
             } else if (arg.type == CCVALTYPE.CODE_POINT) {
-                addCodeRange(env, arg.vs, arg.vs);
+                addCodeRange(env, arg.from, arg.from);
+                if (ascCc != null) ascCc.addCodeRange(env, arg.from, arg.from, false);
             }
             break;
 
         case RANGE:
             if (arg.inType == arg.type) {
                 if (arg.inType == CCVALTYPE.SB) {
-                    if (arg.vs > 0xff || arg.v > 0xff) throw new ValueException(ErrorMessages.ERR_INVALID_CODE_POINT_VALUE);
+                    if (arg.from > 0xff || arg.to > 0xff) throw new ValueException(ErrorMessages.ERR_INVALID_CODE_POINT_VALUE);
 
-                    if (arg.vs > arg.v) {
+                    if (arg.from > arg.to) {
                         if (env.syntax.allowEmptyRangeInCC()) {
                             // goto ccs_range_end
                             arg.state = CCSTATE.COMPLETE;
                             break;
                         } else {
-                            throw new ValueException(ErrorMessages.ERR_EMPTY_RANGE_IN_CHAR_CLASS);
+                            throw new ValueException(ErrorMessages.EMPTY_RANGE_IN_CHAR_CLASS);
                         }
                     }
-                    bs.setRange(arg.vs, arg.v);
+                    bs.setRange(env, arg.from, arg.to);
+                    if (ascCc != null) ascCc.bs.setRange(null, arg.from, arg.to);
                 } else {
-                    addCodeRange(env, arg.vs, arg.v);
+                    addCodeRange(env, arg.from, arg.to);
+                    if (ascCc != null) ascCc.addCodeRange(env, arg.from, arg.to, false);
                 }
             } else {
-                if (arg.vs > arg.v) {
+                if (arg.from > arg.to) {
                     if (env.syntax.allowEmptyRangeInCC()) {
                         // goto ccs_range_end
                         arg.state = CCSTATE.COMPLETE;
                         break;
                     } else {
-                        throw new ValueException(ErrorMessages.ERR_EMPTY_RANGE_IN_CHAR_CLASS);
+                        throw new ValueException(ErrorMessages.EMPTY_RANGE_IN_CHAR_CLASS);
                     }
                 }
-                bs.setRange(arg.vs, arg.v < 0xff ? arg.v : 0xff);
-                addCodeRange(env, arg.vs, arg.v);
+                bs.setRange(env, arg.from, arg.to < 0xff ? arg.to : 0xff);
+                addCodeRange(env, arg.from, arg.to);
+                if (ascCc != null) {
+                    ascCc.bs.setRange(null, arg.from, arg.to < 0xff ? arg.to : 0xff);
+                    ascCc.addCodeRange(env, arg.from, arg.to, false);
+                }
             }
             // ccs_range_end:
             arg.state = CCSTATE.COMPLETE;
@@ -517,13 +484,13 @@ public final class CClassNode extends Node {
 
         } // switch
 
-        arg.vsIsRaw = arg.vIsRaw;
-        arg.vs = arg.v;
+        arg.fromIsRaw = arg.toIsRaw;
+        arg.from = arg.to;
         arg.type = arg.inType;
     }
 
     // onig_is_code_in_cc_len
-    public boolean isCodeInCCLength(int encLength, int code) {
+    boolean isCodeInCCLength(int encLength, int code) {
         boolean found;
 
         if (encLength > 1 || code >= BitSet.SINGLE_BYTE_SIZE) {
@@ -565,24 +532,4 @@ public final class CClassNode extends Node {
     public boolean isNot() {
         return (flags & FLAG_NCCLASS_NOT) != 0;
     }
-
-    public void setShare() {
-        flags |= FLAG_NCCLASS_SHARE;
-    }
-
-    public void clearShare() {
-        flags &= ~FLAG_NCCLASS_SHARE;
-    }
-
-    public boolean isShare() {
-        return (flags & FLAG_NCCLASS_SHARE) != 0;
-    }
-
-    private static int CR_FROM(int[] range, int i) {
-        return range[(i * 2) + 1];
-    }
-    
-    private static int CR_TO(int[] range, int i) {
-        return range[(i * 2) + 2];
-    }
 }
