@@ -25,16 +25,12 @@ public final class CTypeNode extends Node {
     public final boolean asciiRange;
 
     public CTypeNode(int type, boolean not, boolean asciiRange) {
+        super(CTYPE);
         this.ctype= type;
         this.not = not;
         this.asciiRange = asciiRange;
     }
 
-    @Override
-    public int getType() {
-        return CTYPE;
-    }
-
     @Override
     public String getName() {
         return "Character Type";
@@ -44,9 +40,8 @@ public final class CTypeNode extends Node {
     public String toString(int level) {
         StringBuilder value = new StringBuilder();
         value.append("\n  ctype: " + ctype);
-        value.append("\n  not: " + not);
-        value.append("\n  ascii: " + asciiRange);
-
+        value.append(", not: " + not);
+        value.append(", ascii: " + asciiRange);
         return value.toString();
     }
 
