@@ -57,7 +57,7 @@ public interface SyntaxProperties {
     final int OP2_ESC_CAPITAL_Q_QUOTE        = (1<<0);  /* \Q...\E */
     final int OP2_QMARK_GROUP_EFFECT         = (1<<1);  /* (?...); */
     final int OP2_OPTION_PERL                = (1<<2);  /* (?imsxadlu), (?-imsx), (?^imsxalu) */
-    final int OP2_OPTION_RUBY                = (1<<3);  /* (?imx);, (?-imx);  */
+    final int OP2_OPTION_RUBY                = (1<<3);  /* (?imxadu);, (?-imx);  */
     final int OP2_PLUS_POSSESSIVE_REPEAT     = (1<<4);  /* ?+,*+,++ */
     final int OP2_PLUS_POSSESSIVE_INTERVAL   = (1<<5);  /* {n,m}+   */
     final int OP2_CCLASS_SET_OP              = (1<<6);  /* [...&&..[..]..] */
@@ -85,8 +85,10 @@ public interface SyntaxProperties {
     final int OP2_QMARK_BAR_BRANCH_RESET     = (1<<28); /* (?|...) */
     final int OP2_QMARK_LPAREN_CONDITION     = (1<<29); /* (?(cond)yes...|no...) */
     final int OP2_QMARK_CAPITAL_P_NAMED_GROUP= (1<<30); /* (?P<name>...), (?P=name), (?P>name) -- Python/PCRE */
-    final int OP2_OPTION_JAVA                = (1<<31); /* (?idmsux), (?-idmsux) */
-    final int OP2_OPTION_ECMASCRIPT          = (1<<32); /* EcmaScript quirks */
+    final int OP2_QMARK_TILDE_ABSENT         = (1<<31); /* (?~...) */
+
+    final int OP3_OPTION_JAVA                = (1<<0); /* (?idmsux), (?-idmsux) */
+    final int OP3_OPTION_ECMASCRIPT          = (1<<1); /* EcmaScript quirks */
 
     /* syntax (behavior); */
     final int CONTEXT_INDEP_ANCHORS           = (1<<31); /* not implemented */
@@ -110,6 +112,7 @@ public interface SyntaxProperties {
     /* syntax (behavior); warning */
     final int WARN_CC_OP_NOT_ESCAPED          = (1<<24); /* [,-,] */
     final int WARN_REDUNDANT_NESTED_REPEAT    = (1<<25); /* (?:a*);+ */
+    final int WARN_CC_DUP                     = (1<<26); /* [aa] */
 
     final int POSIX_COMMON_OP =
                             OP_DOT_ANYCHAR | OP_POSIX_BRACKET |
